@@ -5,25 +5,38 @@
 
     The CheckExternalLinksBuilder class.
 
-    :copyright: Copyright 2007-2013 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import re
-import sys
-import Queue
 import socket
+import codecs
 import threading
 from os import path
-from urllib2 import build_opener, unquote, Request, \
-    HTTPError, HTTPRedirectHandler
-from HTMLParser import HTMLParser, HTMLParseError
 
+from six.moves import queue
+from six.moves.urllib.request import build_opener, Request, HTTPRedirectHandler
+from six.moves.urllib.parse import unquote
+from six.moves.urllib.error import HTTPError
+from six.moves.html_parser import HTMLParser
 from docutils import nodes
 
+# 2015-06-25 barry@python.org.  This exception was deprecated in Python 3.3 and
+# removed in Python 3.5, however for backward compatibility reasons, we're not
+# going to just remove it.  If it doesn't exist, define an exception that will
+# never be caught but leaves the code in check_anchor() intact.
+try:
+    from six.moves.html_parser import HTMLParseError
+except ImportError:
+    class HTMLParseError(Exception):
+        pass
+
 from sphinx.builders import Builder
+from sphinx.util import encode_uri
 from sphinx.util.console import purple, red, darkgreen, darkgray, \
     darkred, turquoise
+from sphinx.util.pycompat import TextIOWrapper
 
 
 class RedirectHandler(HTTPRedirectHandler):
@@ -43,6 +56,11 @@ opener.addheaders = [('User-agent', 'Mozilla/5.0 (X11; Linux x86_64; rv:25.0) '
 
 class HeadRequest(Request):
     """Subclass of urllib2.Request that sends a HEAD request."""
+    def __init__(self, *args, **kwargs):
+        Request.__init__(self, *args, **kwargs)
+        # we do not parse the response in HEAD, so accepting anything is okay
+        self.headers['Accept-encoding'] = '*'
+
     def get_method(self):
         return 'HEAD'
 
@@ -62,11 +80,11 @@ class AnchorCheckParser(HTMLParser):
                 self.found = True
 
 
-def check_anchor(f, hash):
-    """Reads HTML data from a filelike object 'f' searching for anchor 'hash'.
+def check_anchor(f, anchor):
+    """Reads HTML data from a filelike object 'f' searching for *anchor*.
     Returns True if anchor was found, False otherwise.
     """
-    parser = AnchorCheckParser(hash)
+    parser = AnchorCheckParser(anchor)
     try:
         # Read file in chunks of 8192 bytes. If we find a matching anchor, we
         # break the loop early in hopes not to have to download the whole thing.
@@ -82,6 +100,17 @@ def check_anchor(f, hash):
     return parser.found
 
 
+def get_content_charset(f):
+    content_type = f.headers.get('content-type')
+    if content_type:
+        params = (p.strip() for p in content_type.split(';')[1:])
+        for param in params:
+            if param.startswith('charset='):
+                return param[8:]
+
+    return None
+
+
 class CheckExternalLinksBuilder(Builder):
     """
     Checks for broken external links.
@@ -89,7 +118,7 @@ class CheckExternalLinksBuilder(Builder):
     name = 'linkcheck'
 
     def init(self):
-        self.to_ignore = map(re.compile, self.app.config.linkcheck_ignore)
+        self.to_ignore = [re.compile(x) for x in self.app.config.linkcheck_ignore]
         self.good = set()
         self.broken = {}
         self.redirected = {}
@@ -99,8 +128,8 @@ class CheckExternalLinksBuilder(Builder):
         open(path.join(self.outdir, 'output.txt'), 'w').close()
 
         # create queues and worker threads
-        self.wqueue = Queue.Queue()
-        self.rqueue = Queue.Queue()
+        self.wqueue = queue.Queue()
+        self.rqueue = queue.Queue()
         self.workers = []
         for i in range(self.app.config.linkcheck_workers):
             thread = threading.Thread(target=self.check_thread)
@@ -110,18 +139,82 @@ class CheckExternalLinksBuilder(Builder):
 
     def check_thread(self):
         kwargs = {}
-        if sys.version_info > (2, 5) and self.app.config.linkcheck_timeout:
+        if self.app.config.linkcheck_timeout:
             kwargs['timeout'] = self.app.config.linkcheck_timeout
 
+        def check_uri():
+            # split off anchor
+            if '#' in uri:
+                req_url, anchor = uri.split('#', 1)
+            else:
+                req_url = uri
+                anchor = None
+
+            # handle non-ASCII URIs
+            try:
+                req_url.encode('ascii')
+            except UnicodeError:
+                req_url = encode_uri(req_url)
+
+            try:
+                if anchor and self.app.config.linkcheck_anchors and \
+                   not anchor.startswith('!'):
+                    # Read the whole document and see if #anchor exists
+                    # (Anchors starting with ! are ignored since they are
+                    # commonly used for dynamic pages)
+                    req = Request(req_url)
+                    f = opener.open(req, **kwargs)
+                    encoding = 'utf-8'
+                    if hasattr(f.headers, 'get_content_charset'):
+                        encoding = f.headers.get_content_charset() or encoding
+                    else:
+                        encoding = get_content_charset(f) or encoding
+                    found = check_anchor(TextIOWrapper(f, encoding),
+                                         unquote(anchor))
+                    f.close()
+
+                    if not found:
+                        raise Exception("Anchor '%s' not found" % anchor)
+                else:
+                    try:
+                        # try a HEAD request, which should be easier on
+                        # the server and the network
+                        req = HeadRequest(req_url)
+                        f = opener.open(req, **kwargs)
+                        f.close()
+                    except HTTPError as err:
+                        if err.code != 405:
+                            raise
+                        # retry with GET if that fails, some servers
+                        # don't like HEAD requests and reply with 405
+                        req = Request(req_url)
+                        f = opener.open(req, **kwargs)
+                        f.close()
+            except HTTPError as err:
+                if err.code == 401:
+                    # We'll take "Unauthorized" as working.
+                    return 'working', ' - unauthorized', 0
+                else:
+                    return 'broken', str(err), 0
+            except Exception as err:
+                return 'broken', str(err), 0
+            if f.url.rstrip('/') == req_url.rstrip('/'):
+                return 'working', '', 0
+            else:
+                new_url = f.url
+                if anchor:
+                    new_url += '#' + anchor
+                code = getattr(req, 'redirect_code', 0)
+                return 'redirected', new_url, code
+
         def check():
             # check for various conditions without bothering the network
-            if len(uri) == 0 or uri[0] == '#' or \
-               uri[0:7] == 'mailto:' or uri[0:4] == 'ftp:':
+            if len(uri) == 0 or uri.startswith(('#', 'mailto:', 'ftp:')):
                 return 'unchecked', '', 0
-            elif not (uri[0:5] == 'http:' or uri[0:6] == 'https:'):
+            elif not uri.startswith(('http:', 'https:')):
                 return 'local', '', 0
             elif uri in self.good:
-                return 'working', '', 0
+                return 'working', 'old', 0
             elif uri in self.broken:
                 return 'broken', self.broken[uri], 0
             elif uri in self.redirected:
@@ -130,52 +223,20 @@ class CheckExternalLinksBuilder(Builder):
                 if rex.match(uri):
                     return 'ignored', '', 0
 
-            if '#' in uri:
-                req_url, hash = uri.split('#', 1)
-            else:
-                req_url = uri
-                hash = None
-
             # need to actually check the URI
-            try:
-                if hash and self.app.config.linkcheck_anchors:
-                    # Read the whole document and see if #hash exists
-                    req = Request(req_url)
-                    f = opener.open(req, **kwargs)
-                    found = check_anchor(f, unquote(hash))
-                    f.close()
+            for _ in range(self.app.config.linkcheck_retries):
+                status, info, code = check_uri()
+                if status != "broken":
+                    break
 
-                    if not found:
-                        raise Exception("Anchor '%s' not found" % hash)
-                else:
-                    try:
-                        # try a HEAD request, which should be easier on
-                        # the server and the network
-                        req = HeadRequest(req_url)
-                        f = opener.open(req, **kwargs)
-                        f.close()
-                    except HTTPError, err:
-                        if err.code != 405:
-                            raise
-                        # retry with GET if that fails, some servers
-                        # don't like HEAD requests and reply with 405
-                        req = Request(req_url)
-                        f = opener.open(req, **kwargs)
-                        f.close()
-
-            except Exception, err:
-                self.broken[uri] = str(err)
-                return 'broken', str(err), 0
-            if f.url.rstrip('/') == req_url.rstrip('/'):
+            if status == "working":
                 self.good.add(uri)
-                return 'working', 'new', 0
-            else:
-                new_url = f.url
-                if hash:
-                    new_url += '#' + hash
-                code = getattr(req, 'redirect_code', 0)
-                self.redirected[uri] = (new_url, code)
-                return 'redirected', new_url, code
+            elif status == "broken":
+                self.broken[uri] = info
+            elif status == "redirected":
+                self.redirected[uri] = (info, code)
+
+            return (status, info, code)
 
         while True:
             uri, docname, lineno = self.wqueue.get()
@@ -188,7 +249,7 @@ class CheckExternalLinksBuilder(Builder):
         uri, docname, lineno, status, info, code = result
         if status == 'unchecked':
             return
-        if status == 'working' and info != 'new':
+        if status == 'working' and info == 'old':
             return
         if lineno:
             self.info('(line %4d) ' % lineno, nonl=1)
@@ -198,13 +259,14 @@ class CheckExternalLinksBuilder(Builder):
             self.info(darkgray('-local-   ') + uri)
             self.write_entry('local', docname, lineno, uri)
         elif status == 'working':
-            self.info(darkgreen('ok        ')  + uri)
+            self.info(darkgreen('ok        ')  + uri + info)
         elif status == 'broken':
-            self.info(red('broken    ') + uri + red(' - ' + info))
             self.write_entry('broken', docname, lineno, uri + ': ' + info)
-            if self.app.quiet:
+            if self.app.quiet or self.app.warningiserror:
                 self.warn('broken link: %s' % uri,
                           '%s:%s' % (self.env.doc2path(docname), lineno))
+            else:
+                self.info(red('broken    ') + uri + red(' - ' + info))
         elif status == 'redirected':
             text, color = {
                 301: ('permanently', darkred),
@@ -250,7 +312,7 @@ class CheckExternalLinksBuilder(Builder):
             self.app.statuscode = 1
 
     def write_entry(self, what, docname, line, uri):
-        output = open(path.join(self.outdir, 'output.txt'), 'a')
+        output = codecs.open(path.join(self.outdir, 'output.txt'), 'a', 'utf-8')
         output.write("%s:%s: [%s] %s\n" % (self.env.doc2path(docname, None),
                                            line, what, uri))
         output.close()
